#include "Calculation.h"
#include "MPI.h"

int main(int argc, char **argv) {
	
	// The program started without parameters
	if(argc < 2) {
		fprintf(stdout, "The wrong count of arguments.\n");
		fprintf(stdout, "Using documentation of part 'execute app' or parameter '-h' for showing help\n");
		wait_for_exit();
	}
	
	// The time of calculation
	double t1, t2;


	// The name of input files (bounds, database and parameters)
	char* databaseName = NULL;		
	char* boundsFilename = NULL;	
	char* configFilename = NULL;

	// The database
	sqlite3 *db;

	// Processing all of parameters
	for(int i=1; i < argc;i++) {
		if (strcmp(argv[i], "-h") == 0) {
			// processing argument "HELP"
			show_help();
			wait_for_exit();
		} else if(strcmp(argv[i], "-c") == 0) {
			// processing argument "CONFIG"
			if((i+1) < argc) {
				i++;
				configFilename = argv[i];
			} else {
				fprintf(stdout, "Missing name of config\n");
				wait_for_exit();
			}
		} else if(strcmp(argv[i], "-d") == 0) {
			// processing argument "DATABASE"
			if((i+1) < argc) {
				i++;
				databaseName = argv[i];
			} else {
				fprintf(stdout, "Missing name of database\n");
				wait_for_exit();
			}
		} else if(strcmp(argv[i], "-b") == 0) {
			// processing argument "BOUNDS"
			if((i+1) < argc) {
				i++;
				boundsFilename = argv[i];
			} else {
				fprintf(stdout, "Missing name of bounds\n");
				wait_for_exit();
			}
		} else {
			fprintf(stdout, "Unknown parameter %s\n", argv[i]);
			wait_for_exit();
		}
	}
	
	// start MPI
	int id_node, count_process;

	MPI_Init(&argc, &argv); 

	MPI_Comm_rank(MPI_COMM_WORLD, &id_node);
	MPI_Comm_size(MPI_COMM_WORLD, &count_process);

	// Loading bounds from file
	if(id_node == ROOT_PROCESS) {
		fprintf(stdout, "Loading bounds: %s\n", boundsFilename);
	}
	load_bounds(boundsFilename);

	// Loading parameters from file
	if(id_node == ROOT_PROCESS) {
		fprintf(stdout, "Loading configuration: %s\n", configFilename);
	}
	load_config(configFilename);

	// Loading data from database
	if(id_node == ROOT_PROCESS) {
		fprintf(stdout, "Opening database: %s\n", databaseName);
	}
	if (open_database(&db, databaseName)) {
		load_data_from_db(db);
	} 
	
	
	

	t1 = MPI_Wtime();
	if(id_node == ROOT_PROCESS) {
		//farmer
		fprintf(stdout, "Create new Farmer: %d\n", id_node);
		farmer(db, count_process);
	} else {
		//worker
		fprintf(stdout, "Create new Worker: %d\n", id_node);
		hard_worker(id_node);
	}
	t2 = MPI_Wtime();
	
	MPI_Finalize();

	// clean up after myself
	clean_up(db);
	if(id_node == ROOT_PROCESS) {
		fprintf(stdout, "The mpi part finished in: %.2f\n", t2-t1);
		wait_for_exit();
	}else{
		return 0;
	}
}


