/******************************************************************
The semestral paper of ZCU/KIV/PPR - TBB version

Author: David Kosek A14N0132P
*******************************************************************/

#include "Calculation.h"

/*
Main function, processing parameters and start calculation process for serial and TBB version

parameters: 
int argc	- the count of arguments from command line 
char **argv	- the arguments from command line
*/
int main(int argc, char **argv) {

	// The program started without parameters
	if(argc < 2) {
		fprintf(stdout, "The wrong count of arguments.\n");
		fprintf(stdout, "Using documentation of part 'execute app' or parameter '-h' for showing help\n");
		wait_for_exit();
	}

	// The name of input files (bounds, database and parameters)
	char* databaseName = NULL;		
	char* boundsFilename = NULL;	
	char* configFilename = NULL;

	// The database
	sqlite3 *db;

	// The choice of calculation
	char* executeCalculation = NULL;

	// Processing all of parameters
	for(int i=1; i < argc;i++) {
		if (strcmp(argv[i], "-h") == 0) {
			// processing argument "HELP"
			show_help();
			wait_for_exit();
		} else if(strcmp(argv[i], "-c") == 0) {
			// processing argument "CONFIG"
			if((i+1) < argc) {
				i++;
				configFilename = argv[i];
			} else {
				fprintf(stdout, "Missing name of config\n");
				wait_for_exit();
			}
		} else if(strcmp(argv[i], "-d") == 0) {
			// processing argument "DATABASE"
			if((i+1) < argc) {
				i++;
				databaseName = argv[i];
			} else {
				fprintf(stdout, "Missing name of database\n");
				wait_for_exit();
			}
		} else if(strcmp(argv[i], "-b") == 0) {
			// processing argument "BOUNDS"
			if((i+1) < argc) {
				i++;
				boundsFilename = argv[i];
			} else {
				fprintf(stdout, "Missing name of bounds\n");
				wait_for_exit();
			}
		} else if(strcmp(argv[i], "-e") == 0) {
			// processing argument "EXECUTE" choice
			if((i+1) < argc) {
				i++;
				executeCalculation = argv[i];
			} else {
				fprintf(stdout, "Missing a choice of calculation\n");
				wait_for_exit();
			}
		} else {
			fprintf(stdout, "Unknown parameter %s\n", argv[i]);
			wait_for_exit();
		}
	}

	// Loading bounds from file
	fprintf(stdout, "Loading bounds: %s\n", boundsFilename);
	load_bounds(boundsFilename);

	// Loading parameters from file
	fprintf(stdout, "Loading configuration: %s\n", configFilename);
	load_config(configFilename);

	// Loading data from database
	fprintf(stdout, "Opening database: %s\n", databaseName);
	if (open_database(&db, databaseName)) {
		load_data_from_db(db);
	} 

	// start calculation
	if(strcmp(executeCalculation, "serial") == 0) {
		fprintf(stdout, "Starting serial calculation\n");
		tbb::task_scheduler_init init(1);
		calculation_process(db);	
	} else if(strcmp(executeCalculation, "tbb") == 0) {
		fprintf(stdout, "Starting tbb calculation\n");
		tbb::task_scheduler_init init;
		calculation_process(db);	
	}

	// clean up after myself
	clean_up(db);
	wait_for_exit();
}

