#include "Utils.h"

/*
Convert time from %Y-%m-%dT%H:%M:%S to decimal value

parameters: 
std::string aTime - time in format %Y-%m-%dT%H:%M:%S

return the value for the request time
*/
double convert_time_to_double(std::string aTime) {
	unsigned long long diffFrom1970To1900 = 2209161600000;
	double MSecsPerDay = 24.0*60.0*60.0*1000.0;

	std::tm tm;
	std::stringstream ss(aTime);
	ss >> std::get_time(&tm, "%Y-%m-%dT%H:%M:%S");
	tm.tm_hour -= 1;
	auto tp = std::chrono::system_clock::from_time_t(std::mktime(&tm));

	std::chrono::milliseconds ms = std::chrono::duration_cast< std::chrono::milliseconds >(tp.time_since_epoch());
	unsigned long long diffFrom1900 = diffFrom1970To1900 + ms.count();

	return ((double)diffFrom1900) / MSecsPerDay;
}

/*
Get a maximum time.

parameters:
std::vector<double,std::allocator<double>> aValues - array with values for getting maximum 

return the maximum from array
*/
double get_max_time(std::vector<double,std::allocator<double>> aValues) {
	int length = aValues.size();  // establish size
	double max = aValues[0];       // start with max = first element
	for(int i = 1; i<length; i++) {
		if(aValues[i] > max)
			max = aValues[i];
	}
	return max;   
}

/*
Get a minimun time.

parameters:
std::vector<double,std::allocator<double>> aValues - array with values for getting minimum 

return the minimum from array
*/
double get_min_time(std::vector<double,std::allocator<double>> aValues) {
	int length = aValues.size();  // establish size
	double min = aValues[0];       // start with max = first element
	for(int i = 1; i<length; i++) {
		if(aValues[i] < min)
			min = aValues[i];
	}
	return min;   
}

/*
Show the help
*/
void show_help() {
	fprintf(stdout, "Usage A14N0132P.exe [parameters]\n\n");
	fprintf(stdout, "Parameters\t\t\tNote \n");
	fprintf(stdout, "-d [file name of database]\tCalculation for specified database\n");
	fprintf(stdout, "-b [file name of bounds]\tCalculation for specified bounds\n");
	fprintf(stdout, "-c [file name of parameters]\tCalculation for specified parameters\n");
	fprintf(stdout, "-e [serial | tbb | mpi]\t\tType of execute calculation\n");
	fprintf(stdout, "-h \t\t\t\tShow help\n");
}

/*	
Function for waiting for the end
*/
void wait_for_exit() {
	fprintf(stdout, "Press any key to exit\n");
	std::cin.get();
	exit(0);
}

